importClass(Packages.com.alarmpoint.integrationagent.security.EncryptionUtils);
importClass(Packages.java.util.LinkedHashMap);

var encryptionUtils = new EncryptionUtils(); 

// ----------------------------------------------------------------------------------------------------
// Callbacks requested for this integration service. Add "status" to add event status callback calls.
// ----------------------------------------------------------------------------------------------------
CALLBACKS = ANNOTATE_DELIVERY ? [ "deliveryStatus", "response" ] : [ "response" ];

/*
  Called when response object has been received from xMatters 
*/
function apia_response(apxml) {
    IALOG.info("Calling JavaScript method apia_callback");
    var apxmlAsObj = APXML.toResponseJs(apxml);
    apia_callback(apxmlAsObj);
}

function apia_callback(msg) {
  IALOG.debug("Received message from xMatters:\nIncident: {0}\nEvent Id: {1}\nCallback Type: {2}", msg.incident_id, msg.eventidentifier, msg.xmatters_callback_type);
    
  try {
  
    switch (msg.xmatters_callback_type) {

      case "response":
        try {
          handleResponse(msg);
        } catch (e) {
          handleException(msg, e);
        }
        break;
        
      case "status":  // not used in this integration, provided as a template
        handleEventStatus(msg);
        break;
      
      case "deliveryStatus":
        handleDeliveryStatus(msg);
        break;
    }
    
  } catch (e) {
    IALOG.error("apia_callback(" + msg.eventidentifier + ", " + msg.xmatters_callback_type + "): caught Exception - name: [" + e.name + "], message [" + e.message + "]: " + msg + "\n" + e);

    throw e;
  }    
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * handleDeliveryStatus
 *
 * The main routine for handling delivery status messages from xMatters
 *
 * If an message is delivered successfully, a comment is added to the worklog of the Remedy Incident ticket quoting the user and device that was targeted
 * If an message delivery fails, a comment is added to the worklog of the Remedy Incident ticket quoting the user and device that was targeted
 * ---------------------------------------------------------------------------------------------------------------------
 */
function handleDeliveryStatus(msg) {

  if ( ANNOTATE_DELIVERY && msg.deliverystatus ) {
  
    switch (String(msg.deliverystatus).toLowerCase()) {
    
      case "delivered":
        addAnnotation(getIncidentID(msg), "Notification delivered successfully to " + msg.recipient + " | " + msg.device);
        break;
        
      case "failed": 
        addAnnotation(getIncidentID(msg), "Unable to deliver notification to " + msg.recipient + " | " + msg.device);
        break;
    }
  }
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * handleResponse
 *
 * The main routine for handling user responses
 *
 * ---------------------------------------------------------------------------------------------------------------------
 */
function handleResponse(msg) {
  IALOG.debug("Enter - handleResponse");

  var incidentId = getIncidentID(msg);
  var responder = msg.recipient;
  var device = msg.device;
  var annotation = "null".equals(msg.annotation) ? null : msg.annotation;

  IALOG.info("{0}: Received response '{1}' from {2}|{3} with comment '{4}'", incidentId, msg.response, responder, device, annotation);

  var username = null;
  var password = null;

  if (USE_DEFAULT_CREDENTIALS) {
    IALOG.debug("{0}: using default credentials for HPSM_USER_NAME ({1}) for user {2}", incidentId, HPSM_USER_NAME, responder);  
    username = HPSM_USER_NAME;
    password = HPSM_PASSWORD;
  } else {
    IALOG.debug("{0}: checking user credentials for user {2}", incidentId, responder);  
    // Check if user credentials are set
    var user = getUser(responder);
    
    username = user.customFields.get(HPSM_LOGIN_CUSTOM_FIELD);
    password = user.customFields.get(HPSM_PASSWORD_CUSTOM_FIELD);
    
    if ( username == null || username == "" || password == null || password == "" || password == "[]") {
      // Custom field(s) are not set
      
      if (ENFORCE_CUSTOM_FIELDS) {
        IALOG.warn("{0}: Custom fields '{1}' and/or '{2}' are missing for user {3}, which are both manditory for processing of the response. Response '{4}' is ignored.", 
          incidentId, HPSM_LOGIN_CUSTOM_FIELD, HPSM_PASSWORD_CUSTOM_FIELD, responder, msg.response);
        return;
      } else {
        IALOG.info("{0}: Custom fields '{1}' and/or '{2}' are missing for user {3}, which are both manditory for processing of the response. Will be using default credentials for HPSM_USER_NAME: {4}", 
          incidentId, HPSM_LOGIN_CUSTOM_FIELD, HPSM_PASSWORD_CUSTOM_FIELD, responder, HPSM_USER_NAME);
    
        username = HPSM_USER_NAME;
        password = HPSM_PASSWORD;
      }
    } else {
      IALOG.debug("{0}: using credentials {1} for user {2}", incidentId, user.customFields, username);  
      password = encryptionUtils.decrypt(password);       
    }
  }
 
  switch (String(msg.response).toLowerCase()) {
  
    case RESPONSE_ACTION_RESOLVE:
      // create the connection to the SM Script Object
      var smConnection = ServiceManagerScriptObject( HPSM_URL, username, password );

      smConnection.updateIncident(incidentId, notePrefix + "Resolved by " + responder + " (" + device + ")", "", responder);
      
      if (annotation != null) {            
        smConnection.updateIncident(incidentId, notePrefix + annotation + " --" + responder + " | " + device);
      }
      smConnection.resolveIncident(incidentId, resolutionText, "temporary", null, closureCode);
      
      IALOG.info("{0}: Resolved by {1} | {2}", incidentId, responder, device);
      break;
      
    case RESPONSE_ACTION_OWN:
      // always update status to Work In Progress as default user to avoid event being terminated by HP SM script
      if (username != HPSM_USER_NAME) {
        var sm = ServiceManagerScriptObject( HPSM_URL, HPSM_USER_NAME, HPSM_PASSWORD );
        var fields = new LinkedHashMap();
        fields.put("Status", "Work In Progress");
        sm.updateIncident(incidentId, fields);
      }      
      // create the connection to the SM Script Object
      var smConnection = ServiceManagerScriptObject( HPSM_URL, username, password );
      smConnection.updateIncident(incidentId, notePrefix + "Owned by " + responder + " (" + device + ")", username == HPSM_USER_NAME ? "Work In Progress" : "", responder);
      if (annotation != null) {    
        smConnection.updateIncident(incidentId, notePrefix + annotation + " --" + responder + " | " + device);
      }
      IALOG.info("{0}: Owned by {1} | {2}", incidentId, responder, device);
      break;
      
    case RESPONSE_ACTION_ESCALATE:
    /*
      var smConnection = ServiceManagerScriptObject( HPSM_URL, username, password );
      if (annotation != null) {            
        smConnection.updateIncident(incidentId, notePrefix + annotation + " --" + responder + " | " + device);
      }
    */
      break;
      
    case RESPONSE_ACTION_ACK:
    case RESPONSE_ACTION_ACKNOWLEDGE:  
      // create the connection to the SM Script Object
      var smConnection = ServiceManagerScriptObject( HPSM_URL, username, password );

      smConnection.updateIncident(incidentId, notePrefix + "Acknowledged by " + responder + " (" + device + ")", "", responder);
      
      if (annotation != null) {            
        smConnection.updateIncident(incidentId, notePrefix + annotation + " --" + responder + " | " + device);
      }
      IALOG.info("{0}: Acknowledged by {1} | {2}", incidentId, responder, device);
      break;
      
    default:
      IALOG.error("Unknown response action: '" + msg.response + "'");
  }
  IALOG.debug("Exit - handleResponse");    
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * handleException
 *
 * Notify a responder about an error during the processing of the response 
 *
 * @param msg message
 * @param e exception
 * ---------------------------------------------------------------------------------------------------------------------
 */
function handleException( msg, e ) {
  var errMsg = "Caught exception while processing response from xMatters: " + ( e.name ? e.message : e);
  var incidentId = getIncidentID( msg );
  
  var exceptionMessage;
  
  if ( errMsg.indexOf(LOCKED_RECORD_INDICATOR) > 0 ) {
    IALOG.error("\tEncountered an error processing response '" +  msg.response + "'. Record " + incidentId + " is locked. ");

    var hpsmError = e;
    
    exceptionMessage = "Your response '" + msg.response + "' was not processed as the HP SM record " + incidentId +
      " is locked. To record your response, log into HP Service Manager and update the incident manually. \n\nHP Service Manager returned the following error message:\n\n" + hpsmError;

  } else {
    IALOG.error(errMsg);
    
    exceptionMessage = "HP Service Manager could not process the request and returned the following error message:\n\n " 
      + errMsg.replace(exceptionString, "") + ".\n\nTo resolve this issue, please contact your HP Service Manager Administrator."
  }
  
  try {
    // Create APXML
    var apxml = ServiceAPI.createAPXML();

    apxml.setToken(ERROR_MESSAGE_BODY, exceptionMessage);
    apxml.setToken(ERROR_MESSAGE_SUBJECT, "Your response '" + msg.response + "' for HP SM record " + incidentId + " was not processed");
    apxml.setToken(ERROR_MESSAGE_RESPONSE, msg.response);
    apxml.setToken("recipients", "[{\"targetName\":\"" + msg.recipient + "\"}]");  // expected by APXML.toJSON as an array of objects
    apxml.setToken("xmpriority", ERROR_MESSAGE_PRIORITY);
    apxml.setToken("agent_client_id", ServiceAPI.getConfiguration().getName());

    // Send event
    IALOG.debug("Sending error notification to [" + msg.recipient + "]");
    var response = XMRESTAPI.submitApxml(ERROR_FORM, apxml, null, null, null);
    
    if (response && response.status == XMRESTAPI.RESPONSE_SUCCESS) {
      IALOG.info("Error notification was sent to [" + msg.recipient + "]");
    } else if (response) {
      IALOG.error("Error notification was not sent to " + msg.recipient + ". xMatters returned response (" + response.status + ")");      
    } else {
      IALOG.error("Error notification was not sent to " + msg.recipient + " because of deduplication settings");      
    }
  } catch (e) {
    IALOG.error("Error notification was not sent to " + msg.recipient + " due to exception: " + e);      
  }  
}

/**
 * ---------------------------------------------------------------------------------------------------------------------
 * handleEventStatus 
 *
 * The template for handling event status messages from xMatters
 * ---------------------------------------------------------------------------------------------------------------------
 */
function handleEventStatus(msg)
{
  switch (msg.status) {
 
    case "active":     
      // "Event " + msg.eventidentifier + " successfully created in xMatters"
      break;
      
    case "terminated": 
      // "Event " + msg.eventidentifier + " terminated"
      break;
      
    case "terminated_external": // event terminated by user if msg.username != INITIATOR
      if (msg.username !== INITIATOR) {
        // "Event " + msg.eventidentifier + " manually terminated from within xMatters" /* + " by user [" + msg.username + "]" */ 
      } else {
        // Ignore the events created and terminated by this integration and not the actual user
      }
      break;
  }
}